#[cfg(feature = "Wdk_System_IO")]
#[doc = "Required features: `\"Wdk_System_IO\"`"]
pub mod IO;
#[cfg(feature = "Wdk_System_OfflineRegistry")]
#[doc = "Required features: `\"Wdk_System_OfflineRegistry\"`"]
pub mod OfflineRegistry;
#[cfg(feature = "Wdk_System_Registry")]
#[doc = "Required features: `\"Wdk_System_Registry\"`"]
pub mod Registry;
#[cfg(feature = "Wdk_System_SystemInformation")]
#[doc = "Required features: `\"Wdk_System_SystemInformation\"`"]
pub mod SystemInformation;
#[cfg(feature = "Wdk_System_SystemServices")]
#[doc = "Required features: `\"Wdk_System_SystemServices\"`"]
pub mod SystemServices;
#[cfg(feature = "Wdk_System_Threading")]
#[doc = "Required features: `\"Wdk_System_Threading\"`"]
pub mod Threading;
