/* Overlay controls for next, zoom, fullscreen */
button.osd.circular {
  min-width: 42px;
  min-height: 42px;
}

.fullscreen revealer:not(.raised) .fullscreen-osd {
  color: rgba(255, 255, 255, 0.9);
  background: rgba(0, 0, 0, 0.65);
}

shadow,
border,
outline {
  background: none;
}

.drag-icon {
  margin: 16px;
  border-radius: 5px;
  outline: 2px solid white;
  box-shadow: 0 0 10px rgba(0, 0, 0, 200);
}

/* Remove default adwaita style */
.drop-widget:drop(active):focus,
.drop-widget:drop(active) {
  box-shadow: none;
}

/* Drag overlay */
.lp-dragging-area-highlight {
  border-radius: 8px;
  background-color: color-mix(in srgb, var(--accent-bg-color) 35%, transparent);
  margin: 24px;
}

lpprintpreview {
  background-color: #858585;
  padding: 30px;
}

lpprintpreviewpage {
  background-color: #f0f0f0;
  box-shadow: 0 0 14px 3px #666;
}

lpprintpreviewpage lpimage {
  outline: 1px dotted rgba(0, 0, 0, 0.3);
}

dropdown.flat button:not(:hover) {
  background: none;
}

dialog.error-details textview {
  background: none;
  color: inherit;
}

.bottom-properties {
  box-shadow: 0 -1px var(--headerbar-darker-shade-color);
}

.error-message {
  padding: 12px;
  font-size: 0.95em;
}
